package com.gic.haoban.manage.service.service.chat.impl;

import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.stream.Collectors;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.BasePageInfo;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.commons.util.UniqueIdUtils;
import com.gic.haoban.manage.api.constants.Manage3Constants;
import com.gic.haoban.manage.api.dto.SecretSettingDTO;
import com.gic.haoban.manage.api.dto.WxEnterpriseQwDTO;
import com.gic.haoban.manage.api.dto.chat.GroupChatHmDTO;
import com.gic.haoban.manage.api.dto.qdto.chat.ChatHmSearchQDTO;
import com.gic.haoban.manage.api.enums.SecretTypeEnum;
import com.gic.haoban.manage.service.dao.mapper.chat.GroupChatHmMapper;
import com.gic.haoban.manage.service.dao.mapper.chat.GroupChatHmRelationMapper;
import com.gic.haoban.manage.service.dao.mapper.chat.GroupChatMapper;
import com.gic.haoban.manage.service.entity.chat.TabGroupChat;
import com.gic.haoban.manage.service.entity.chat.TabGroupChatHm;
import com.gic.haoban.manage.service.entity.chat.TabGroupChatHmRelation;
import com.gic.haoban.manage.service.pojo.bo.chat.GroupChatHmBO;
import com.gic.haoban.manage.service.service.SecretSettingService;
import com.gic.haoban.manage.service.service.WxEnterpriseService;
import com.gic.haoban.manage.service.service.chat.GroupChatHmService;
import com.gic.wechat.api.dto.qywx.chat.AddJoinWayDTO;
import com.gic.wechat.api.service.qywx.QywxChatApiService;
import com.github.pagehelper.PageHelper;

@Service("groupChatHmService")
public class GroupChatHmServiceImpl implements GroupChatHmService {

    private static final Logger logger = LoggerFactory.getLogger(GroupChatHmService.class);

    @Autowired
    private QywxChatApiService qywxChatApiService;
    @Autowired
    private WxEnterpriseService wxEnterpriseService;
    @Autowired
    private SecretSettingService secretSettingService;
    @Autowired
    private GroupChatMapper groupChatMapper;
    @Autowired
    private GroupChatHmMapper groupChatHmMapper;
    @Autowired
    private GroupChatHmRelationMapper groupChatHmRelationMapper;

    @Override
    public Page<GroupChatHmBO> listPage(ChatHmSearchQDTO qdto, BasePageInfo basePageInfo) {
        PageHelper.startPage(basePageInfo);
        String wxEnterpriseId = qdto.getWxEnterpriseId();
        String enterpriseId = qdto.getEnterpriseId();
        Date startDate = qdto.getStartDate();
        Date endDate = qdto.getEndDate();
        Integer statusFlag = qdto.getStatus();
        List<TabGroupChatHm> list = this.groupChatHmMapper.list(wxEnterpriseId, enterpriseId, startDate, endDate,
                qdto.getSearchParams(), statusFlag);
        Page<GroupChatHmBO> retPage = PageHelperUtils.changePageHelperToCurrentPage(list, GroupChatHmBO.class);
        return retPage;
    }

    @Override
    public GroupChatHmBO getById(Long hmId) {
        TabGroupChatHm hm = this.groupChatHmMapper.selectById(hmId);
        GroupChatHmBO bo = EntityUtil.changeEntityByJSON(GroupChatHmBO.class, hm);
        return bo;
    }

    @Override
    public void saveHm(GroupChatHmDTO dto) {
        String wxEnterpriseId = dto.getWxEnterpriseId();
        WxEnterpriseQwDTO qwDTO = this.wxEnterpriseService.getQwInfo(wxEnterpriseId);
        SecretSettingDTO secretSetting = secretSettingService.getSecretSetting(wxEnterpriseId,
                SecretTypeEnum.CUSTOMIZED_APP.getVal());
        if (null == secretSetting) {
            logger.info("没有配置代开应用");
        }
        boolean updateFlag = false;
        Long hmid = UniqueIdUtils.uniqueLong();
        if (null != dto.getChatHmId()) {
            updateFlag = true;
            hmid = dto.getChatHmId();
        } else {
            hmid = UniqueIdUtils.uniqueLong();
        }
        dto.setChatHmId(hmid);
        dto.setChatHmCode("HM10" + hmid);
        List<Long> chatidList = dto.getChatIdList() ;
        List<TabGroupChat> chatList = this.groupChatMapper.listByIdSet(new HashSet<>(chatidList));
        List<String> wxChatidList = chatList.stream().map(item -> item.getWxChatId()).collect(Collectors.toList());
        logger.info("企微群id={}", wxChatidList);
        AddJoinWayDTO joinDTO = new AddJoinWayDTO();
        joinDTO.setScene(2);
        joinDTO.setRemark(dto.getName());
        joinDTO.setAuto_create_room(dto.getAutoCreateRoom() == null ? 0 : dto.getAutoCreateRoom());
        joinDTO.setRoom_base_id(dto.getRoomBaseId() == null ? 0 : dto.getRoomBaseId());
        joinDTO.setRoom_base_name(dto.getRoomBaseName());
        joinDTO.setState(Manage3Constants.QHM + hmid);
        joinDTO.setChat_id_list(wxChatidList);

        TabGroupChatHm hm = EntityUtil.changeEntityByJSON(TabGroupChatHm.class, dto);
        hm.setDeleteFlag(0);
        hm.setCreateTime(new Date());
        hm.setUpdateTime(new Date());
        hm.setChatCount(wxChatidList.size());

        if (updateFlag) {
            joinDTO.setConfig_id(hm.getWxConfigId());
            ServiceResponse<Void> updateResp = this.qywxChatApiService.updateJoinWay(qwDTO.getDkCorpid(),
                    secretSetting.getSecretVal(), joinDTO);
            if (updateResp.isSuccess()) {
                this.groupChatHmMapper.updateById(hm);
            }
        } else {
            ServiceResponse<String> addResp = this.qywxChatApiService.addJoinWay(qwDTO.getDkCorpid(),
                    secretSetting.getSecretVal(), joinDTO);
            logger.info("addResp={}", JSON.toJSONString(addResp));
            String configId = addResp.getResult();

            ServiceResponse<AddJoinWayDTO> getResp = this.qywxChatApiService.getJoinWay(qwDTO.getDkCorpid(),
                    secretSetting.getSecretVal(), configId);
            logger.info("getResp={}", JSON.toJSONString(getResp));
            String qrCode = getResp.getResult().getQr_code();
            hm.setWxConfigId(configId);
            hm.setWxQrCode(qrCode);
            this.groupChatHmMapper.insert(hm);
            // 保存realtion
            for(TabGroupChat item : chatList) {
                TabGroupChatHmRelation entity = EntityUtil.changeEntity(TabGroupChatHmRelation.class, item) ;
                entity.setChatHmId(hm.getChatHmId());
                entity.setRelationId(UniqueIdUtils.uniqueLong());
                this.groupChatHmRelationMapper.insert(entity) ;
            }
        }
    }

}