package com.gic.haoban.manage.service.service.chat.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.BasePageInfo;
import com.gic.api.base.commons.JSONResponse;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.clerk.api.dto.ClerkDTO;
import com.gic.clerk.api.service.ClerkService;
import com.gic.commons.util.DateUtil;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.commons.util.UniqueIdUtils;
import com.gic.enterprise.api.dto.StoreDTO;
import com.gic.enterprise.api.service.StoreService;
import com.gic.haoban.manage.api.dto.WxEnterpriseQwDTO;
import com.gic.haoban.manage.api.dto.chat.GroupChatPlanDTO;
import com.gic.haoban.manage.api.dto.chat.GroupChatPlanLogDTO;
import com.gic.haoban.manage.api.dto.chat.GroupChatPlanOwnerLogDTO;
import com.gic.haoban.manage.api.dto.qdto.chat.GroupChatPlanSearchQDTO;
import com.gic.haoban.manage.service.config.Config;
import com.gic.haoban.manage.service.dao.mapper.StaffMapper;
import com.gic.haoban.manage.service.dao.mapper.chat.GroupChatMapper;
import com.gic.haoban.manage.service.dao.mapper.chat.GroupChatPlanLogMapper;
import com.gic.haoban.manage.service.dao.mapper.chat.GroupChatPlanMapper;
import com.gic.haoban.manage.service.entity.TabHaobanStaff;
import com.gic.haoban.manage.service.entity.chat.TabGroupChat;
import com.gic.haoban.manage.service.entity.chat.TabGroupChatPlan;
import com.gic.haoban.manage.service.entity.chat.TabGroupChatPlanLog;
import com.gic.haoban.manage.service.entity.chat.TabGroupChatPlanOwnerLog;
import com.gic.haoban.manage.service.pojo.bo.chat.GroupChatBO;
import com.gic.haoban.manage.service.pojo.bo.chat.GroupChatPlanBO;
import com.gic.haoban.manage.service.service.QywxSendService;
import com.gic.haoban.manage.service.service.WxEnterpriseService;
import com.gic.haoban.manage.service.service.chat.GroupChatPlanOwnerLogService;
import com.gic.haoban.manage.service.service.chat.GroupChatPlanService;
import com.gic.haoban.manage.service.service.chat.GroupChatService;
import com.gic.wechat.api.dto.qywx.QywxSendMessageResultDTO;
import com.gic.wechat.api.service.qywx.QywxSuiteApiService;
import com.github.pagehelper.PageHelper;

@Service("groupChatPlanService")
public class GroupChatPlanServiceImpl implements GroupChatPlanService {

    private static org.apache.logging.log4j.Logger logger = LogManager.getLogger(GroupChatPlanService.class);

    @Autowired
    private GroupChatPlanMapper groupChatPlanMapper;
    @Autowired
    private GroupChatPlanOwnerLogService groupChatPlanOwnerLogService;
    @Autowired
    private GroupChatPlanLogMapper groupChatPlanLogMapper;
    @Autowired
    private QywxSendService qywxSendService;
    @Autowired
    private WxEnterpriseService wxEnterpriseService;
    @Autowired
    private Config config;
    @Autowired
    private QywxSuiteApiService qywxSuiteApiService;
    @Autowired
    private StaffMapper staffMapper;
    @Autowired
    private GroupChatService groupChatService;
    @Autowired
    private ClerkService clerkService;
    @Autowired
    private StoreService storeService;
    @Autowired
    private GroupChatMapper groupChatMapper;

    @Override
    public void save(GroupChatPlanDTO dto) {
        TabGroupChatPlan entity = EntityUtil.changeEntityByJSON(TabGroupChatPlan.class, dto);
        if (null == dto.getPlanId()) {
            entity.setPlanId(UniqueIdUtils.uniqueLong());
            this.groupChatPlanMapper.insert(entity);
        } else {
            this.groupChatPlanMapper.updateById(entity);
        }
    }

    @Override
    public GroupChatPlanBO getById(Long planId) {
        TabGroupChatPlan plan = this.groupChatPlanMapper.selectById(planId);
        GroupChatPlanBO bo = EntityUtil.changeEntityByJSON(GroupChatPlanBO.class, plan);
        return bo;
    }

    @Override
    public Page<GroupChatPlanBO> listPage(GroupChatPlanSearchQDTO qdto, BasePageInfo basePageInfo) {
        PageHelper.startPage(basePageInfo);
        List<TabGroupChatPlan> list = this.groupChatPlanMapper.list(qdto);
        Page<GroupChatPlanBO> retPage = PageHelperUtils.changePageHelperToCurrentPage(list, GroupChatPlanBO.class);
        return retPage;
    }

    @Override
    public void doPlan(Long planId) {
        logger.info("开始执行群群发计划,planId={}", planId);
        TabGroupChatPlan plan = this.groupChatPlanMapper.selectById(planId);
        if (null == plan) {
            logger.info("计划不存在,planid={}", planId);
            return;
        }
        String[] staffIdList = plan.getStaffIdList().split(",");
        String wxEnterpriseId = plan.getWxEnterpriseId();
        String enterpriseId = plan.getEnterpriseId();
        String chatContent = plan.getChatContent();
        JSONArray jsonArr = JSONArray.parseArray(chatContent);
        List<String> madidList = new ArrayList<>();
        for (int i = 0; i < jsonArr.size(); i++) {
            JSONObject json = jsonArr.getJSONObject(i);
            madidList.add(json.getString("relation_id"));
        }
        for (int i = 0; i < staffIdList.length; i++) {
            String staffId = staffIdList[i];
            ServiceResponse<String> sendResp = qywxSendService.sendChatMessage(wxEnterpriseId, staffId, madidList);
            GroupChatPlanOwnerLogDTO ownerDTO = new GroupChatPlanOwnerLogDTO();
            if (sendResp.isSuccess()) {
                String msgid = sendResp.getResult();
                ownerDTO.setMsgid(msgid);
                ownerDTO.setSendStatus(1);
            } else {
                ownerDTO.setRemark(sendResp.getMessage());
                ownerDTO.setSendStatus(0);
            }
            // 群主群数量
            ownerDTO.setExpireDate(DateUtil.addDay(new Date(), plan.getExpireDays()));
            ownerDTO.setTotalCount(0);
            ownerDTO.setWxEnterpriseId(wxEnterpriseId);
            ownerDTO.setEnterpriseId(enterpriseId);
            ownerDTO.setStaffId(staffId);
            ownerDTO.setPlanId(planId);
            this.groupChatPlanOwnerLogService.save(ownerDTO);
        }
    }

    @Override
    public void doPlanSendInfoTimer(String params) {
        Long owner_log_id = 472092751301763073L;
        TabGroupChatPlanOwnerLog ownerLog = this.groupChatPlanOwnerLogService.getById(owner_log_id);
        String wxEnterpriseId = ownerLog.getWxEnterpriseId();
        String enterpriseId = ownerLog.getEnterpriseId();
        String msgid = ownerLog.getMsgid();
        WxEnterpriseQwDTO qwDTO = this.wxEnterpriseService.getQwInfo(wxEnterpriseId);
        if (qwDTO != null) {
        }

        String staffId = ownerLog.getStaffId();

        TabHaobanStaff staff = this.staffMapper.selectByPrimaryKey(staffId);
        String userid = staff.getWxUserId();
        if (qwDTO.needOpenUserId3th()) {
            userid = staff.getWxOpenUseId();
        }

        JSONResponse respon = qywxSuiteApiService.resultExternalMessage(qwDTO.getThirdCorpid(), config.getWxSuiteid(),
                msgid, null, userid);
        logger.info(JSON.toJSONString(respon));
        if (respon.getErrorCode() == 0) {
            List<QywxSendMessageResultDTO> list = (List<QywxSendMessageResultDTO>) respon.getList();
            if (CollectionUtils.isNotEmpty(list)) {
                int sendCount = list.size();
                this.groupChatPlanOwnerLogService.updateSendCount(ownerLog.getOwnerLogId(), sendCount,
                        list.get(0).getSendTime());
                for (QywxSendMessageResultDTO item : list) {
                    String wxChatId3th = item.getChatId();
                    Date sendTime = item.getSendTime();
                    TabGroupChatPlanLog entity = new TabGroupChatPlanLog();
                    entity.setLogId(UniqueIdUtils.uniqueLong());
                    entity.setPlanId(ownerLog.getPlanId());
                    entity.setOwnLogId(ownerLog.getOwnerLogId());
                    entity.setWxEnterpriseId(wxEnterpriseId);
                    entity.setEnterpriseId(enterpriseId);
                    entity.setWxChatId3th(wxChatId3th);
                    GroupChatBO chat = this.groupChatService.getByWxChatId3th(wxEnterpriseId, wxChatId3th);
                    if (null != chat) {
                        entity.setGroupChatId(chat.getGroupChatId());
                    }
                    entity.setSendTime(sendTime);
                    entity.setSendStatus(1);
                    this.groupChatPlanLogMapper.insert(entity);
                }

            }
        }
    }

    @Override
    public Page<GroupChatPlanLogDTO> listLogPage(GroupChatPlanSearchQDTO qdto, BasePageInfo basePageInfo) {
        PageHelper.startPage(basePageInfo);
        List<GroupChatPlanLogDTO> list = this.groupChatPlanLogMapper.listPlanLog(qdto);
        if (CollectionUtils.isNotEmpty(list)) {
            Set<String> clerkIdList = list.stream().filter(dto -> StringUtils.isNotBlank(dto.getClerkId()))
                    .map(dto -> dto.getClerkId()).collect(Collectors.toSet());
            if (CollectionUtils.isNotEmpty(clerkIdList)) {
                List<ClerkDTO> clerkList = this.clerkService.getClerkByClerkIds(clerkIdList);
                if (CollectionUtils.isNotEmpty(clerkList)) {
                    Map<String, ClerkDTO> map = clerkList.stream()
                            .collect(Collectors.toMap(ClerkDTO::getClerkId, dto -> dto, (k1, k2) -> k1));
                    list.forEach(dto -> {
                        String clerkId = dto.getClerkId();
                        if (StringUtils.isNotBlank(clerkId) && null != map.get(clerkId)) {
                            dto.setClerkCode(map.get(clerkId).getClerkCode());
                            dto.setClerkName(map.get(clerkId).getClerkName());
                        }
                    });
                }
            }
            List<String> storeIdList = list.stream().filter(dto -> StringUtils.isNotBlank(dto.getStoreId()))
                    .map(dto -> dto.getStoreId()).collect(Collectors.toList());
            if (CollectionUtils.isNotEmpty(storeIdList)) {
                String[] arr = (String[]) storeIdList.toArray();
                List<StoreDTO> storeList = this.storeService.getStores(arr);
                Map<String, StoreDTO> map = storeList.stream()
                        .collect(Collectors.toMap(StoreDTO::getStoreId, dto -> dto, (k1, k2) -> k1));
                list.forEach(dto -> {
                    String storeId = dto.getStoreId();
                    if (StringUtils.isNotBlank(storeId) && null != map.get(storeId)) {
                        dto.setStoreCode(map.get(storeId).getStoreCode());
                        dto.setStoreName(map.get(storeId).getStoreName());
                    }
                });
            }

            Set<Long> groupChatIdSet = list.stream().map(dto -> dto.getGroupChatId()).collect(Collectors.toSet());
            List<TabGroupChat> chatList = this.groupChatMapper.listByIdSet(groupChatIdSet);
            if (CollectionUtils.isNotEmpty(chatList)) {
                Map<Long, TabGroupChat> map = chatList.stream()
                        .collect(Collectors.toMap(TabGroupChat::getGroupChatId, dto -> dto, (k1, k2) -> k1));
                list.forEach(dto -> {
                    Long groupChatId = dto.getGroupChatId();
                    if (null != groupChatId && null != map.get(groupChatId)) {
                        dto.setChatName(map.get(groupChatId).getName());
                        dto.setChatTotalCount(map.get(groupChatId).getTotalCount());
                    }
                });
            }
        }
        Page<GroupChatPlanLogDTO> retPage = PageHelperUtils.changePageHelperToCurrentPage(list,
                GroupChatPlanLogDTO.class);
        return retPage;
    }

}